@@ -4,7 +4,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2017 Slaven Rezic. All rights reserved.
+# Copyright (C) 2017,2018 Slaven Rezic. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -17,7 +17,7 @@ use warnings;
 
 {
     package Doit;
-    our $VERSION = '0.025_51';
+    our $VERSION = '0.025_52';
 
     use constant IS_WIN => $^O eq 'MSWin32';
 }
@@ -1004,23 +1004,44 @@ use warnings;
     sub cmd_system {
 	my($self, @args) = @_;
 	my %options; if (@args && ref $args[0] eq 'HASH') { %options = %{ shift @args } }
-	@args = Doit::Win32Util::win32_quote_list(@args) if Doit::IS_WIN;
+	my $quiet = delete $options{quiet};
+	my $info = delete $options{info};
 	my $show_cwd = delete $options{show_cwd};
 	error "Unhandled options: " . join(" ", %options) if %options;
+
+	@args = Doit::Win32Util::win32_quote_list(@args) if Doit::IS_WIN;
+
+	my $code = sub {
+	    system @args;
+	    if ($? != 0) {
+		_handle_dollar_questionmark;
+	    }
+	};
+
 	my @commands;
 	push @commands, {
-			 code => sub {
-			     system @args;
-			     if ($? != 0) {
-				 _handle_dollar_questionmark;
-			     }
-			 },
-			 msg  => "@args" . _show_cwd($show_cwd),
-			 rv   => 1,
+			 ($info
+			  ? (
+			     rv   => do { $code->(); 1 },
+			     code => sub {},
+			    )
+			  : (
+			     rv   => 1,
+			     code => $code,
+			    )
+			 ),
+			 ($quiet ? () : (msg  => "@args" . _show_cwd($show_cwd))),
 			};
 	Doit::Commands->new(@commands);
     }
 
+    sub cmd_info_system {
+	my($self, @args) = @_;
+	my %options; if (@args && ref $args[0] eq 'HASH') { %options = %{ shift @args } }
+	$options{info} = 1;
+	$self->cmd_system(\%options, @args);
+    }
+
     sub cmd_touch {
 	my($self, @files) = @_;
 	my @commands;
@@ -1602,13 +1623,14 @@ use warnings;
     }
 
     for my $cmd (
-		 qw(chmod chown mkdir rename rmdir symlink system unlink utime),
+		 qw(chmod chown mkdir rename rmdir symlink unlink utime),
 		 qw(make_path remove_tree), # File::Path
 		 qw(copy move), # File::Copy
 		 qw(run), # IPC::Run
 		 qw(qx info_qx), # qx// and variant which even runs in dry-run mode, both using list syntax
 		 qw(open2 info_open2), # IPC::Open2
 		 qw(open3 info_open3), # IPC::Open3
+		 qw(system info_system), # builtin system with variant
 		 qw(cond_run), # conditional run
 		 qw(touch), # like unix touch
 		 qw(ln_nsf), # like unix ln -nsf
@@ -1710,7 +1732,19 @@ use warnings;
 		if ($self->{debug}) {
 		    info "Reaping process $pid...";
 		}
-		my $got_pid = waitpid $pid, &POSIX::WNOHANG;
+		my $start_time = time;
+		my $got_pid = Doit::RPC::gentle_retry(
+		    code => sub {
+			waitpid $pid, &POSIX::WNOHANG;
+		    },
+		    retry_msg_code => sub {
+			my($seconds) = @_;
+			if (time - $start_time >= 2) {
+			    info "can't reap process $pid, sleep for $seconds seconds";
+			}
+		    },
+		    fast_sleep => 0.01,
+		);
 		if (!$got_pid) {
 		    warning "Could not reap process $pid...";
 		}
@@ -1754,7 +1788,7 @@ use warnings;
 
 {
     package Doit::RPC::Client;
-    use vars '@ISA'; @ISA = ('Doit::RPC');
+    our @ISA = ('Doit::RPC');
 
     sub new {
 	my($class, $infh, $outfh, %options) = @_;
@@ -1794,7 +1828,7 @@ use warnings;
 
 {
     package Doit::RPC::Server;
-    use vars '@ISA'; @ISA = ('Doit::RPC');
+    our @ISA = ('Doit::RPC');
 
     sub new {
 	my($class, $runner, $sockpath, %options) = @_;
@@ -1872,7 +1906,7 @@ use warnings;
 
 {
     package Doit::RPC::SimpleServer;
-    use vars '@ISA'; @ISA = ('Doit::RPC');
+    our @ISA = ('Doit::RPC');
     
     sub new {
 	my($class, $runner, $infh, $outfh, %options) = @_;
@@ -1920,7 +1954,7 @@ use warnings;
 	$self->{rpc}->call_remote(@args);
     }
 
-    use vars '$AUTOLOAD';
+    our $AUTOLOAD;
     sub AUTOLOAD {
 	(my $method = $AUTOLOAD) =~ s{.*::}{};
 	my $self = shift;
@@ -1957,7 +1991,7 @@ use warnings;
 {
     package Doit::Sudo;
 
-    use vars '@ISA'; @ISA = ('Doit::_AnyRPCImpl');
+    our @ISA = ('Doit::_AnyRPCImpl');
 
     use Doit::Log;
 
@@ -2046,7 +2080,7 @@ use warnings;
 {
     package Doit::SSH;
 
-    use vars '@ISA'; @ISA = ('Doit::_AnyRPCImpl');
+    our @ISA = ('Doit::_AnyRPCImpl');
 
     use Doit::Log;
 
@@ -2211,7 +2245,9 @@ use warnings;
 
     sub DESTROY {
 	my $self = shift;
+	local $?; # XXX Net::OpenSSH::_waitpid sets $?=0
 	if ($self->{ssh}) {
+	    $self->{ssh}->disconnect if $self->{ssh}->can('disconnect');
 	    delete $self->{ssh};
 	}
 	if ($self->{rpc}) {
