@@ -3,7 +3,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2017 Slaven Rezic. All rights reserved.
+# Copyright (C) 2017,2018 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -15,9 +15,7 @@ package Doit::Git; # Convention: all commands here should be prefixed with 'git_
 
 use strict;
 use warnings;
-
-use vars qw($VERSION);
-$VERSION = '0.023';
+our $VERSION = '0.024';
 
 use Doit::Log;
 use Doit::Util qw(in_directory);
@@ -33,6 +31,7 @@ sub git_repo_update {
     my @repository_aliases = @{ delete $opts{repository_aliases} || [] };
     my $directory  = delete $opts{directory};
     my $origin     = delete $opts{origin} || 'origin';
+    my $branch     = delete $opts{branch};
     my $allow_remote_url_change = delete $opts{allow_remote_url_change};
     my $clone_opts = delete $opts{clone_opts};
     my $refresh    = delete $opts{refresh} || 'always';
@@ -72,24 +71,46 @@ sub git_repo_update {
 			"or specify allow_remote_url_change=>1\n";
 		}
 	    }
-	    if ($refresh eq 'always') {
-		if ($quiet) {
-		    # XXX there's no quiet option for system, misuse qx instead
-		    $self->qx({quiet=>1}, qw(git fetch));
-		} else {
-		    $self->system({show_cwd=>1}, qw(git fetch));
+
+	    # In some cases a "git fetch" is not sufficient to fetch
+	    # from the expected remote, e.g. for a detached branch.
+	    # Extra heuristics are used to fill @origin_for_fetch
+	    # in these situations.
+	    my @origin_for_fetch;
+	    if (defined $branch) { # maybe branch switching necessary?
+		my %info;
+		my $current_branch = $self->git_current_branch(info_ref => \%info);
+		if (!defined $current_branch || $current_branch ne $branch) {
+		    $self->system({show_cwd=>1,quiet=>$quiet}, qw(git checkout), $branch);
+		    $has_changes = 1;
+		    %info = ();
+		    $self->git_current_branch(info_ref => \%info);
 		}
+		if ($info{detached} && $branch =~ m{^(.*?)/}) {
+		    @origin_for_fetch = $1;
+		}
+	    }
+
+	    if ($refresh eq 'always') {
+		$self->system({show_cwd=>1,quiet=>$quiet}, qw(git fetch), @origin_for_fetch);
 		my $status = $self->git_short_status(untracked_files => 'no');
 		if ($status =~ m{>$}) {
 		    # may actually fail if diverged (status=<>)
 		    # or untracked/changed files would get overwritten
-		    $self->system({show_cwd=>1}, qw(git pull)); # XXX actually would be more efficient to do a merge or rebase, but need to figure out how git does it exactly...
+		    $self->system({show_cwd=>1,quiet=>$quiet}, qw(git pull)); # XXX actually would be more efficient to do a merge or rebase, but need to figure out how git does it exactly... # XXX does not work if @origin_for_fetch is set
 		    $has_changes = 1;
 		} # else: ahead, diverged, or something else
+
+		if (@origin_for_fetch) {
+		    $self->system({show_cwd=>1,quiet=>$quiet}, qw(git checkout), $branch);
+		}
 	    }
 	} $directory;
     } else {
 	my @cmd = (qw(git clone --origin), $origin);
+	if (defined $branch) {
+	    push @cmd, "--branch", $branch;
+	}
 	if ($clone_opts) {
 	    push @cmd, @$clone_opts;
 	}
@@ -283,12 +304,60 @@ sub git_is_shallow {
 sub git_current_branch {
     my($self, %opts) = @_;
     my $directory = delete $opts{directory};
+    my $info_ref  = delete $opts{info_ref};
     error "Unhandled options: " . join(" ", %opts) if %opts;
 
     in_directory {
 	my $git_root = $self->git_root;
 	my $fh;
-	open $fh, "<", "$git_root/.git/HEAD" and $_ = <$fh> and m{refs/heads/(\S+)} and return $1;
+	my $this_head;
+	if (open $fh, "<", "$git_root/.git/HEAD") {
+	    chomp($this_head = <$fh>);
+	    if ($this_head =~ m{refs/heads/(\S+)}) {
+		return $1;
+	    }
+	}
+
+	# fallback to git-status
+	$ENV{LC_ALL} = 'C';
+	if (open $fh, '-|', 'git', 'status') {
+	    chomp($_ = <$fh>);
+	    if (/^On branch (.*)/) {
+		if ($info_ref) {
+		    $info_ref->{fallback} = 'git-status';
+		}
+		return $1;
+	    }
+	    if (/^.* detached at (.*)/) {
+		if ($info_ref) {
+		    $info_ref->{detached} = 1;
+		    $info_ref->{fallback} = 'git-status';
+		}
+		return $1;
+	    }
+	    if (/^\Q# Not currently on any branch./) {
+		# Probably old git (~ 1.5 ... 1.7)
+		if (open my $fh2, '-|', 'git', 'show-ref') {
+		    while(<$fh2>) {
+			chomp;
+			if (my($sha1, $ref) = $_ =~ m{^(\S+)\s+refs/remotes/(.*)$}) {
+			    if ($sha1 eq $this_head) {
+				if ($info_ref) {
+				    $info_ref->{detached} = 1;
+				    $info_ref->{fallback} = 'git-show-ref';
+				}
+				return $ref;
+			    }
+			}
+		    }
+		    close $fh2
+			or warning "Problem while running 'git show-ref': $!";
+		} else {
+		    warning "Error running 'git show-ref': $!";
+		}
+	    }
+	}
+
 	undef;
     } $directory;
 }
